local version = "0.4";
local isDebug = false;

local ADDON_NAME = "TankMastery";
local addon = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "LibSink-2.0", "AceConsole-3.0")
local L	= LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)
local media = LibStub("LibSharedMedia-3.0") 
local SOUND = media.MediaType.SOUND
local count

local chanNames = {
  "Disabled",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
	"< Empty >",
}

local defaults = {
	profile = {
		sinkoptions = { sink20OutputSink = "ChatFrame"; },
		sound = nil,
--    raidicon = false,
    outputChannel = 1,
	},
	realm = {
		tauntStats = { ["*"] = { ["*"] = 0 } }
	}
}
local options = {
    name = L["addon_name"],
    desc = L["addon_desc"],
    type = "group",
    get = "GetValue",
    set = "SetValue", 
    handler = addon,
    args = {
	    	output = addon:GetSinkAce3OptionsDataTable(),
	
		    sound = {
				    name = L["sound_name"],
				    desc = L["sound_desc"],
						type = "select",
				    order = 10,
						dialogControl = 'LSM30_Sound',
						values = AceGUIWidgetLSMlists.sound,
				},
--[[	Disabled for now...	
				raidicon = {
				    name = L["raidicon_name"],
				    desc = L["raidicon_desc"],
				    type = "toggle",
				    order = 20,
				},				
--]]
				outputChannel = {
					type = "select",
					name = "Secondary Output Channel",
					width = "150",
					order = 30,
					values = chanNames,
				},
    },
}
options.args.output.inline = true

local DO = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(ADDON_NAME, {
		type = "data source",
		text = L["addon_name"],
        icon = [[Interface\ICONS\Ability_Physical_Taunt]],
})

local taunts = {
	355,   -- Warrior: Taunt
	1161,  -- Warrior: Challenging Shout
	5209,  -- Druid: Challanging Roar
	6795,  -- Druid: Growl
	27270, -- VoidWalker: Torment Rank 7 
	31789, -- Paladin: Righteous Defense
	33700, -- Felguard: Anguish Rank 3
	33701, -- VoidWalker: Suffering Rank 6
	47984, -- VoidWalker: Torment Rank 8
	47989, -- VoidWalker: Suffering Rank 7
	47990, -- VoidWalker: Suffering Rank 8
	47993, -- Felguard: Anguish Rank 4
	56222, -- DK: Dark Command
	59671, -- Warlock: Challanging Howl
	62124, -- Paladin: Hand of Reckoning
}
local mocks = {	
	694,   -- Warrior: Mocking Blow
	49576, -- DK: Death Grip
	20736, -- Hunter: Distracting Shot
}
local knockbacks = {
	53227, -- Druid: Typhoon
	59159, -- Shaman: Thunderstorm
}
local buffRighteousFury = GetSpellInfo(25780)
local buffFrostPresence = GetSpellInfo(48263)
local ICON_LIST = {  
	"{star}",  
	"{circle",  
	"{diamond}",  
	"{triangle}",  
	"{moon}",  
	"{square}",  
	"{cross}",  
	"{skull}",
}

local isTankMasteryEnabled = false;
local isMonitoringTaunts = false;

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("TankMasteryDB", defaults, "Default")
	self:SetSinkStorage(self.db.profile.sinkoptions)
	local acreg = LibStub("AceConfigRegistry-3.0")
	acreg:RegisterOptionsTable(ADDON_NAME, options)
	local acdia = LibStub("AceConfigDialog-3.0")	
	acdia:AddToBlizOptions(ADDON_NAME, ADDON_NAME)
	addon:RegisterChatCommand(ADDON_NAME:lower(), function() acdia:Open(ADDON_NAME) end)
end

function addon:OnEnable()	
		-- only register events if we can tank... 			   
  local isTank = self:isPossibleTank("player");
  if (isTank > 0) then
	  		isTankMasteryEnabled = true
				TankMasteryFrame:UnregisterAllEvents();
	  		media.UnregisterAllCallbacks(self)
				
				TankMasteryFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
				
				-- selective registering (optimization)
				if (isTank == 1) then
					TankMasteryFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
				elseif (isTank == 2) then
					TankMasteryFrame:RegisterEvent("UNIT_AURA");
				end			

				-- channel listing...	  			  		
				TankMasteryFrame:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE");
				addon:populateChanNames(); -- once to support reloadui as well...
					
				ChatFrame1:AddMessage("|cffffff00TankMastery Loaded (v".. version ..")...");
												
				-- startup init
				self:updateTankingMode()
	else
		    isTankMasteryEnabled = false
				ChatFrame1:AddMessage("|cffffff00TankMastery NOT Loaded. We are not a tanking class.");
	end	
	
	TankMasteryFrame:Hide();		
end

function addon:populateChanNames()
    for i=1,9 do
        channelNum, channelName = GetChannelName(i);
        if (channelNum > 0 and channelName ~= nil) then
        	if (string.sub(chanNames[i + 1], 1, 1) == "<") then
            chanNames[i + 1] =  i..". ".. channelName;
          end
        end
    end
end

function TankMasteryFrame_OnEvent(event, ...)
	-- deduce event type	
	if (event == "UPDATE_SHAPESHIFT_FORM") then
		addon:updateTankingMode();
	elseif (event == "UNIT_AURA" and arg1 == "player" ) then
		addon:updateTankingMode();
	elseif (event == "CHAT_MSG_CHANNEL_NOTICE") then
		if (arg7 == 0) then
			chanNames[tonumber(arg8) + 1] = arg4;
		else
			chanNames[tonumber(arg8) + 1] = arg4 .. " (!!!)";
		end
  elseif (event == "COMBAT_LOG_EVENT_UNFILTERED" and isMonitoringTaunts == true) then
		addon:TankMasteryFrame_CombatLogEvent(event, ...);
  end
end

function addon:TankMasteryFrame_CombatLogEvent(event, ...)
	local timestamp, ttype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellid, spellname, spellschool, b_or_d = select(1, ...);
	local tauntmsg = "";
	local isAOE = false;
	local targetName = "";

	if (ttype == "SPELL_CAST_SUCCESS")
		and (UnitGUID("targettarget")) and (UnitGUID("target")) and (UnitGUID("targettarget") == UnitGUID("player")) 
		and (UnitGUID("player") ~= srcGUID)
		and (not CombatLog_Object_IsA(dstFlags, COMBATLOG_OBJECT_TYPE_PLAYER)) then			
			if ((dstName == nil or dstName == "") and UnitName("target")) then			
				self:debugPrint("Non-targeted spell: ".. spellname);
				dstName = UnitName("target");
				isAOE = true;
			end
			targetName = dstName;
			
			if (UnitInParty("player") == 1 or UnitInRaid("player") == 1) then
		  	  if GetRaidTargetIndex("target") then
			  		targetName = ICON_LIST[GetRaidTargetIndex("target")].. dstName;
			  	end
			end
			
			-- taunts etc
			if (self:TableContainsValue(taunts, spellid)) then
				if (not isAOE) then -- only my target
					if (UnitGUID("target") == dstGUID) then						
						tauntmsg = srcName.." taunted "..targetName.." using "..spellname..".";
					end
				else
					tauntmsg = srcName.." taunted "..targetName.." using AOE "..spellname..".";
				end
			end

			-- mocks etc
			if (self:TableContainsValue(mocks, spellid)) then
				tauntmsg = srcName.." mocked "..targetName.." using "..spellname..".";
			end	

			-- knockback etc
			if (self:TableContainsValue(knockbacks, spellid)) then
				tauntmsg = srcName.." knocked-away "..targetName.." using "..spellname..".";
			end	

			if (tauntmsg ~= "") then		
--				  tauntStats = self.db.realm.tauntStats
				
--			    if self.db.profile.raidicon then 
--			        if UnitInRaid("player") and (IsRaidLeader() or IsRaidOfficer()) then
--			        	if not GetRaidTargetIndex(name) then
--			        		SetRaidTarget(name, ICON_TAG_LIST[ICON_TAG_RAID_TARGET_MOON1:lower()])
--			        		self.lastplayer = name
--			        	end
--			        end
--			    end				
				  addon:Pour(tauntmsg);
				  
				  outChan = self.db.profile.outputChannel;
				  if (outChan > 1 and chanNames[outChan] ~= "" and (string.sub(chanNames[i + 1], 1, 1) ~= "<")) then
				  	SendChatMessage(tauntmsg, "CHANNEL", nil, chanNames[outChan]);
				  end
				  
			  	if addon.db.profile.sound then
						PlaySoundFile(media:Fetch(SOUND, addon.db.profile.sound))
					end
			end
	end
end

local LTT = LibStub:GetLibrary( "LibQTip-1.0" )
local tooltip
local function Color(c, t) return "|cff"..c..t.."|r" end
local function HGreen(t) return Color("80ff80", t) end
local function HYellow(t) return Color("ffffa0", t) end
local function Green(t) return Color("80ff80", t) end
local function Blue(t) return Color("8080ff", t) end
local function Red(t) return Color("ff8080", t) end
local function WhiteParens(t) return "|cffffffff(|r"..t.."|cffffffff)|r" end

local function toggleOptionsWindow()
    local acd = LibStub("AceConfigDialog-3.0")
    if acd.OpenFrames[ADDON_NAME] then
        acd:Close(ADDON_NAME)
    else
        acd:Open(ADDON_NAME)
    end
end
function DO.OnClick(frame, button)
	if button == "RightButton" then
        toggleOptionsWindow()
	end
end
function DO.OnEnter(self)
  tooltip = LTT:Acquire(ADDON_NAME, 2, "LEFT", "RIGHT")
  tooltip:Clear()
  if (isTankMasteryEnabled == true) then
  	tooltip:AddHeader(HGreen(ADDON_NAME));
  else
  	tooltip:AddHeader(Red(ADDON_NAME));
  end
	tooltip:AddLine(L["tt_clickhint3L"].." "..L["tt_clickhint3R"])
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo( self )
  tooltip:Show()
end
function DO.OnLeave(self)
    tooltip = LTT:Release(tooltip)
end

-- #FUNCTIONS
function addon:ClearFailIcon()
    if addon.lastplayer then 
        SetRaidTarget(self.lastplayer, 0)
        addon.lastplayer = nil
    end
end

function addon:debugPrint(msg)
	if (isDebug == true) then
		ChatFrame1:AddMessage("DEBUG : |cFF00FF00" .. msg);
	end	
end

function addon:TableContainsValue(table, value)
      if (table and value) then
              for _, v in ipairs(table) do
                      if (v == value) then
                              return true;
                      end
              end
              return false;
      end
end

-- #TANKINGSTATUS
function addon:updateTankingMode()
		if (self:isCurrentlyTanking("player") == true) then 
			self:debugPrint("Tanking...");
			isMonitoringTaunts = true;
		else
			self:debugPrint("NOT Tanking...");
			isMonitoringTaunts = false;
	  end
end
function addon:isPossibleTank(sTarget)
	local PlayerClass = UnitClass(sTarget);
	if PlayerClass == "Druid" then 
		return 1;
	end
	if PlayerClass == "Warrior" then 
		return 1;
	end
	if PlayerClass == "Paladin" then 
		return 2;
	end
	if PlayerClass == "Death Knight" then 
		return 1;
	end 
	return 0;
end
function addon:isCurrentlyTanking(sTarget)
	local PlayerClass = UnitClass(sTarget);
	-- Bear form
	if PlayerClass == "Druid" and GetShapeshiftForm(true) == 1 then 
		return true;
	end
	-- Defensive stance
	if PlayerClass == "Warrior" and GetShapeshiftForm(true) == 2 then 
		return true;
	end
	-- Righteous Fury 
	if PlayerClass == "Paladin" and UnitBuff("player", buffRighteousFury) then 
		return true;
	end
	-- Frost Presence  UnitBuff("player", buffFrostPresence) 
	if PlayerClass == "Death Knight" and GetShapeshiftForm(false) == 2 then 
		return true;
	end

	return false;
end

-- #OPTIONS
function addon:SetValue(info, b)
	self.db.profile[info[#info]] = b
end
function addon:GetValue(info)
	return self.db.profile[info[#info]]
end